package gui;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.SourceDataLine;
import javax.sound.sampled.UnsupportedAudioFileException;
import javax.swing.JOptionPane;
import javax.swing.SwingWorker;

/**
 *Play_Audio Class
 */
class Clip implements LoadableResource
{
	private static Map<URL, Clip> cache = new HashMap<URL, Clip>();
	
	private AudioFormat 		audioFormat;
	private boolean             stopPlayback;
	private URL					soundFile;
	private int					fileSize;
	private int					claimedSize;
	private byte				soundData[];
	private URLConnection 		URLCon;
	private boolean				isURLValid;
	private Boolean				isReadyFlag;
	private double				length = -1;

	public double play()
	{
		waitUntilReady();
		new Player().start(); //play thread
		return length;
	}

	/* (non-Javadoc)
	 * @see gui.LoadableResource#waitUntilReady()
	 */
	public synchronized boolean waitUntilReady() {
		if(isReadyFlag) return true;
		else if(!isURLValid) return false;
		while(!isReadyFlag&&isURLValid) {
			try {
				wait();
			} catch (InterruptedException e) {}
		}
		if(isReadyFlag) return true;
		else return false;

	}
	
	/* (non-Javadoc)
	 * @see gui.LoadableResource#isReady()
	 */
	public boolean isReady() { return isReadyFlag;}
	/* (non-Javadoc)
	 * @see gui.LoadableResource#isValid()
	 */
	public boolean isValid() { return isURLValid;}
	public double getLength() { return length;}
	public void stop() {stopPlayback=true;}

	/* (non-Javadoc)
	 * @see gui.LoadableResource#progress()
	 */
	public double progress() {
		if(claimedSize>0)
			return fileSize/(double)claimedSize;
		else
			return -1;
	}

	public static void preCache(final String base, final Vector<String> list) {
		new Thread() {			
			public void run() {		
				for(String s:list) {
					try {
						Clip newClip = getClip(new URL(base+s));
						newClip.waitUntilReady();
					} catch (MalformedURLException e) {
						e.printStackTrace();
					}
				}
			};
		}.start();
	}
	
	public static void preCache(Vector<String> list) {
		preCache("", list);
	}
	
	public static Clip getClip(URL url) {
		synchronized (cache) {
			if(cache.containsKey(url))
				return cache.get(url);
			else {
				Clip newClip = new Clip(url);
				cache.put(url, newClip);
				return newClip;
			}
		}

			
	}

	private Clip(URL soundFile) {
		this.soundFile=soundFile;		
		fileSize = 0;
		isURLValid = true; isReadyFlag = false;
		new Thread() {
			public void run() {
				try {
					URLCon = Clip.this.soundFile.openConnection();
					claimedSize = URLCon.getContentLength();
					byte dataBuffer[];
					if(claimedSize>0)
						dataBuffer = new byte[claimedSize];
					else {
						dataBuffer = new byte[10000000];
					}
					int increment = 0;
					while(increment>=0) {
						try {
							increment = URLCon.getInputStream().read(dataBuffer,fileSize,dataBuffer.length-fileSize);
						} catch (IOException e) {
							isURLValid = false;
							e.printStackTrace();
						}
						fileSize+=increment;
					}
					soundData = Arrays.copyOf(dataBuffer, fileSize);
					AudioInputStream    audioInputStream;
					audioInputStream = AudioSystem.getAudioInputStream(new ByteArrayInputStream(soundData));
					audioFormat      = audioInputStream.getFormat(); //get format of stream


					if(audioFormat.getEncoding() != AudioFormat.Encoding.PCM_SIGNED) //check that stream is proper format
					{
						audioInputStream = AudioSystem.getAudioInputStream(AudioFormat.Encoding.PCM_SIGNED,audioInputStream);
						audioFormat       = audioInputStream.getFormat(); //fix improper wave format
					}

					DataLine.Info dataLineInfo = new DataLine.Info(SourceDataLine.class,audioFormat); //request a data line (not open)

					AudioSystem.getLine(dataLineInfo);
					stopPlayback   = false; //allow playback
					isReadyFlag = true;
					length = fileSize / (audioFormat.getFrameRate() * audioFormat.getChannels() * audioFormat.getSampleSizeInBits() / 8);
				} catch (Exception e) {
					isURLValid = false;
					e.printStackTrace();
				} //inform line of data type
				synchronized (Clip.this) {
					Clip.this.notifyAll();
				}
			};
		}.start();
	}

	public class Player extends Thread {
		/**
		 *Thread event which handles audio playback
		 */
		public void run()
		{
			byte buffer[] = new byte[10000]; //10KB buffer, will not be too large for any system

			AudioInputStream audioInputStream;
			try {
				audioInputStream = AudioSystem.getAudioInputStream(new ByteArrayInputStream(soundData));

				DataLine.Info dataLineInfo = new DataLine.Info(SourceDataLine.class,audioFormat); //request a data line (not open)

				SourceDataLine sourceDataLine = (SourceDataLine)AudioSystem.getLine(dataLineInfo);
				sourceDataLine.open(audioFormat); //open the line
				sourceDataLine.start(); //prepare line for data transfer

				int continue_play;
				while((continue_play = audioInputStream.read(buffer,0,buffer.length)) != -1 && stopPlayback == false)
				{
					if(continue_play > 0) //data still left to write
					{
						sourceDataLine.write(buffer, 0, continue_play); //write data to the line
					}
				}
				sourceDataLine.drain(); //clear buffer
				sourceDataLine.close(); //close line
			}
			catch(Exception e)
			{
				JOptionPane.showMessageDialog(null, "The audio file cannot be played!"+"\n" + e.getMessage(), 
						"Error!", JOptionPane.INFORMATION_MESSAGE);

			}
		}
	}
}

