package gui;

import java.awt.CardLayout;
import java.awt.EventQueue;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.LinkedList;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.SwingUtilities;

public class ResourceWaiterButton extends JPanel implements ActionListener{

	JButton button;
	JProgressBar progressBar;
	LoadableResource resource;
	Timer timer;
	CardLayout layout;
	boolean enabled;
	UpdateAppearanceTask updateObject;
	List<ActionListener> actionListeners = new LinkedList<ActionListener>();
	String loadingText;
	
	public ResourceWaiterButton(String text) {
		super();
		loadingText = "Loading the clip...";
		generateComponents(text);
	}
	
	public ResourceWaiterButton(String text, String loading) {
		super();
		loadingText = loading;
		generateComponents(text);
	}
		
	public void setResource(LoadableResource clipIn) {
		this.resource = clipIn;
		timer.cancel();
		timer = new Timer();
		timer.schedule(new UpdateAppearanceTask() ,200,200);
		updateObject.run();

	}
	
	public void setText(String text) {
		button.setText(text);
		
	}
	
	private class UpdateAppearanceTask extends TimerTask {
		@Override
		public void run() {
			if(EventQueue.isDispatchThread()) {
				if(!enabled || resource == null)
					layout.show(ResourceWaiterButton.this, "button");
				else if(resource.isReady()) {
					layout.show(ResourceWaiterButton.this, "button");
					timer.cancel();
				} else {
					layout.show(ResourceWaiterButton.this, "bar");
					if(!resource.isValid()) {
						progressBar.setString("The clip is not available");
						timer.cancel();
					} else if(resource.progress()<0) {
						progressBar.setIndeterminate(true);		
					} else {
						progressBar.setIndeterminate(false);		
						progressBar.setValue((int)(resource.progress()*100));
					}
				}		
			}
			else SwingUtilities.invokeLater(this);
		}
	}
	
	private void generateComponents(String text) {
	
		updateObject = new UpdateAppearanceTask();
		enabled = true;
		timer= new Timer();
		button = new JButton(text);
		progressBar = new JProgressBar(0, 100);
		setLayout(layout = new CardLayout());
		add(button, "button");
		add(progressBar, "bar");
		progressBar.setString(loadingText);
		progressBar.setStringPainted(true);
		button.addActionListener(this);
	}
	
	public void addActionListener(ActionListener l){
		actionListeners.add(l);
	}
	
	public void setEnabled(boolean enabled) {
		this.enabled=enabled;
		button.setEnabled(enabled);
		updateObject.run();
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		e.setSource(this);
		for(ActionListener l: actionListeners) {
			l.actionPerformed(e);
		}
		
	}
}
