package gui;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

public class TextFile implements LoadableResource{
	
	private static Map<URL, TextFile> cache = new HashMap<URL, TextFile>();
	
	private byte 				soundData[];
	private int					fileSize;
	private int					claimedSize;
	private URLConnection 		URLCon;
	private boolean				isURLValid;
	private Boolean				isReadyFlag;
	private URL					soundFile;

	public synchronized boolean waitUntilReady() {
		if(isReadyFlag) return true;
		else if(!isURLValid) return false;
		while(!isReadyFlag&&isURLValid) {
			try {
				wait();
			} catch (InterruptedException e) {}
		}
		if(isReadyFlag) return true;
		else return false;

	}
	
	public static TextFile getTextFile(URL url) {
		synchronized (cache) {
			if(cache.containsKey(url))
				return cache.get(url);
			else {
				TextFile newClip = new TextFile(url);
				cache.put(url, newClip);
				return newClip;
			}
		}		
	}
	
	public BufferedReader getBufferedReader() {
		waitUntilReady();
		return new BufferedReader(new InputStreamReader(new ByteArrayInputStream(soundData)));
	}
	
	public static void preCache(final String base, final Vector<String> list) {
		new Thread() {			
			public void run() {		
				for(String s:list) {
					try {
						TextFile newClip = getTextFile(new URL(base+s));
						newClip.waitUntilReady();
					} catch (MalformedURLException e) {
						e.printStackTrace();
					}
				}
			};
		}.start();
	}
	
	public static void preCache(Vector<String> list) {
		preCache("", list);
	}
	
	public boolean isReady() { return isReadyFlag;}
	public boolean isValid() { return isURLValid;}
	
	private TextFile(URL soundFile) {
		this.soundFile=soundFile;		
		fileSize = 0;
		isURLValid = true; isReadyFlag = false;
		new Thread() {
			public void run() {
				try {
					URLCon = TextFile.this.soundFile.openConnection();
					claimedSize = URLCon.getContentLength();
					byte dataBuffer[];
					if(claimedSize>0)
						dataBuffer = new byte[claimedSize];
					else {
						dataBuffer = new byte[10000000];
					}
					int increment = 0;
					while(increment>=0) {
						try {
							increment = URLCon.getInputStream().read(dataBuffer,fileSize,dataBuffer.length-fileSize);
						} catch (IOException e) {
							isURLValid = false;
							e.printStackTrace();
						}
						fileSize+=increment;
					}
					soundData = Arrays.copyOf(dataBuffer, fileSize);
					isReadyFlag = true;
				} catch (Exception e) {
					isURLValid = false;
					e.printStackTrace();
				} //inform line of data type
				synchronized (TextFile.this) {
					TextFile.this.notifyAll();
				}
			};
		}.start();
	}

	public double progress() {
		if(claimedSize>0)
			return fileSize/(double)claimedSize;
		else
			return -1;
	}
}
